@@ -1,37 +1,31 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: format_atmhdr.c 1508 2010-02-03 21:36:53Z salcock $
  *
  */
-
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 
 #include "config.h"
 #include "common.h"
@@ -41,7 +35,6 @@
 #include "wandio.h"
 
 #include <sys/stat.h>
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -144,7 +137,7 @@ static int atmhdr_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	if ((numbytes=wandio_read(libtrace->io, buffer, (size_t)12)) != 12)
 	{
 		if (numbytes != 0) {
-			trace_set_err(libtrace,errno,"read(%s)",libtrace->uridata);
+			trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",libtrace->uridata);
 		}
 		return numbytes;
 	}
@@ -192,7 +185,7 @@ static uint64_t atmhdr_get_erf_timestamp(const libtrace_packet_t *packet) {
 
 static struct libtrace_format_t atmhdr = {
 	"atmhdr",
-	"$Id: format_atmhdr.c 1508 2010-02-03 21:36:53Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_ATMHDR,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -208,7 +201,9 @@ static struct libtrace_format_t atmhdr = {
         atmhdr_read_packet,             /* read_packet */
         atmhdr_prepare_packet,		/* prepare_packet */
 	NULL,                           /* fin_packet */
+        NULL,                           /* can_hold_packet */
         NULL,                           /* write_packet */
+        NULL,                           /* flush_output */
         atmhdr_get_link_type,        	/* get_link_type */
         NULL,                           /* get_direction */
         NULL,                           /* set_direction */
@@ -216,6 +211,7 @@ static struct libtrace_format_t atmhdr = {
         NULL,                           /* get_timeval */
 	NULL,				/* get_timespec */
         NULL,                           /* get_seconds */
+	NULL,				/* get_meta_section */
         NULL,                           /* seek_erf */
         NULL,                           /* seek_timeval */
         NULL,                           /* seek_seconds */
@@ -226,11 +222,12 @@ static struct libtrace_format_t atmhdr = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
         NULL,                           /* get_fd */
         trace_event_trace,              /* trace_event */
         NULL,                 		/* help */
-        NULL                            /* next pointer */
+        NULL,                            /* next pointer */
+	NON_PARALLEL(false)
 };
 	
 
