@@ -1,36 +1,28 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: format_dag24.c 1508 2010-02-03 21:36:53Z salcock $
  *
  */
-
 #define _GNU_SOURCE
 
 #include "config.h"
@@ -40,12 +32,12 @@
 #include "format_helper.h"
 #include "format_erf.h"
 
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 
 #include <sys/mman.h>
 #ifdef WIN32
@@ -185,7 +177,7 @@ static int dag_init_input(libtrace_t *libtrace) {
 		free(dag_dev_name);
                 return -1;
         }
-	
+
 	dag_init_format_data(libtrace);
 	if (S_ISCHR(buf.st_mode)) {
                 /* DEVICE */
@@ -235,8 +227,22 @@ static int dag_config_input(libtrace_t *libtrace, trace_option_t option,
 			 * filtering */
                         return -1;
 		case TRACE_OPTION_EVENT_REALTIME:
+		case TRACE_OPTION_REPLAY_SPEEDUP:
 			/* Live capture is always going to be realtime */
 			return -1;
+		case TRACE_OPTION_HASHER:
+			/* Lets just say we did this, it's currently still up to
+			 * the user to configure this correctly. */
+			return 0;
+                case TRACE_OPTION_CONSTANT_ERF_FRAMING:
+                        return -1;
+		case TRACE_OPTION_DISCARD_META:
+		case TRACE_OPTION_XDP_HARDWARE_OFFLOAD:
+		case TRACE_OPTION_XDP_SKB_MODE:
+		case TRACE_OPTION_XDP_DRV_MODE:
+		case TRACE_OPTION_XDP_ZERO_COPY_MODE:
+		case TRACE_OPTION_XDP_COPY_MODE:
+			return -1;
         }
 	return -1;
 }
@@ -332,7 +338,11 @@ static dag_record_t *dag_get_record(libtrace_t *libtrace) {
 	if (!erfptr)
                 return NULL;
         size = ntohs(erfptr->rlen);
-        assert( size >= dag_record_size );
+	if (size < dag_record_size) {
+                fprintf(stderr, "DAG2.4 rlen is invalid (rlen = %u, must be at least %u)\n",
+			size, dag_record_size);
+                return NULL;
+        }
         FORMAT_DATA->offset += size;
         FORMAT_DATA->diff -= size;
         return erfptr;
@@ -471,6 +481,7 @@ static libtrace_eventobj_t trace_event_dag(libtrace_t *trace,
                                 event.type = TRACE_EVENT_PACKET;
                         } else {
                                 /* Do not sleep - try to read another packet */
+                                trace->filtered_packets ++;
 				continue;
                         }
                 } else {
@@ -481,13 +492,10 @@ static libtrace_eventobj_t trace_event_dag(libtrace_t *trace,
                 if (trace->snaplen > 0) {
                         trace_set_capture_length(packet, trace->snaplen);
                 }
-
+                trace->accepted_packets ++;
                 return event;
         } while (1);
 
-
-	/* We only want to sleep for a very short time */
-        assert(data == 0);
         event.type = TRACE_EVENT_SLEEP;
         event.seconds = 0.0001;
         event.size = 0;
@@ -504,7 +512,7 @@ static uint64_t dag_get_dropped_packets(libtrace_t *trace)
 
 /* Prints some semi-useful help text about the DAG format module */
 static void dag_help(void) {
-        printf("dag format module: $Revision: 1508 $\n");
+        printf("dag format module: $Revision: 1715 $\n");
         printf("Supported input URIs:\n");
         printf("\tdag:/dev/dagn\n");
         printf("\n");
@@ -517,7 +525,7 @@ static void dag_help(void) {
 
 static struct libtrace_format_t dag = {
         "dag",
-        "$Id: format_dag24.c 1508 2010-02-03 21:36:53Z salcock $",
+        "$Id$",
         TRACE_FORMAT_ERF,
 	dag_probe_filename,		/* probe filename */
 	NULL,				/* probe magic */
@@ -533,7 +541,9 @@ static struct libtrace_format_t dag = {
         dag_read_packet,                /* read_packet */
         dag_prepare_packet,		/* prepare_packet */
 	NULL,                           /* fin_packet */
+        NULL,                           /* can_hold_packet */
         NULL,                           /* write_packet */
+        NULL,                           /* flush_output */
         erf_get_link_type,              /* get_link_type */
         erf_get_direction,              /* get_direction */
         erf_set_direction,              /* set_direction */
@@ -544,6 +554,7 @@ static struct libtrace_format_t dag = {
         NULL,                           /* seek_erf */
         NULL,                           /* seek_timeval */
         NULL,                           /* seek_seconds */
+	NULL,                           /* get_meta_section */
         erf_get_capture_length,         /* get_capture_length */
         erf_get_wire_length,            /* get_wire_length */
         erf_get_framing_length,         /* get_framing_length */
@@ -551,11 +562,12 @@ static struct libtrace_format_t dag = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	dag_get_dropped_packets,	/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
         NULL,                           /* get_fd */
         trace_event_dag,                /* trace_event */
         dag_help,                       /* help */
-        NULL                            /* next pointer */
+        NULL,                            /* next pointer */
+    NON_PARALLEL(true)
 };
 
 void dag_constructor(void) {
