@@ -1,36 +1,28 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: format_helper.c 1695 2011-03-10 03:24:22Z salcock $
  *
  */
-
 #include "config.h"
 #include <sys/types.h>
 #include <fcntl.h> /* for O_LARGEFILE */
@@ -45,8 +37,12 @@
 #include <errno.h>
 #include <time.h>
 #include "format_helper.h"
+#include <unistd.h>
+#ifndef _SC_NPROCESSORS_ONLN
+#include <sys/param.h>
+#include <sys/sysctl.h>
+#endif
 
-#include <assert.h>
 #include <stdarg.h>
 
 #ifdef WIN32
@@ -76,9 +72,17 @@ struct libtrace_eventobj_t trace_event_device(struct libtrace_t *trace,
 	int max_fd;
 	struct timeval tv;
 
-	assert(trace != NULL);
-	assert(packet != NULL);
-	
+	if (!trace) {
+		fprintf(stderr, "NULL trace passed into trace_event_device()\n");
+		event.type = TRACE_EVENT_TERMINATE;
+		return event;
+	}
+	if (!packet) {
+		trace_set_err(trace, TRACE_ERR_NULL_PACKET, "NULL packet passed into trace_event_device()");
+		event.type = TRACE_EVENT_TERMINATE;
+		return event;
+	}
+
 	FD_ZERO(&rfds);
 	FD_ZERO(&rfds_param);
 
@@ -97,7 +101,7 @@ struct libtrace_eventobj_t trace_event_device(struct libtrace_t *trace,
 	 * uses mmapped sockets by default, I've switched over to this 
 	 * solution. */
 
-	do {
+        do {
 		tv.tv_sec = 0;
 		tv.tv_usec = 0;
 		rfds_param = rfds;
@@ -110,8 +114,7 @@ struct libtrace_eventobj_t trace_event_device(struct libtrace_t *trace,
 	} while (ret == -1);
 
 	if (FD_ISSET(event.fd, &rfds_param)) {
-		event.size = trace_read_packet(trace,packet);
-		
+                event.size = trace_read_packet(trace,packet);
 		if (event.size < 1) {
 			/* Covers error and EOF events - terminate rather 
 			 * than report a packet as available */
@@ -135,6 +138,9 @@ struct libtrace_eventobj_t trace_event_trace(struct libtrace_t *trace, struct li
 	struct libtrace_eventobj_t event = {0,0,0.0,0};
 	double ts;
 	double now;
+        double sincebeginnow = 0;
+        double sincebegintrace = 0;
+
 #ifdef WIN32
 	struct __timeb64 tstruct;
 #else
@@ -190,16 +196,16 @@ struct libtrace_eventobj_t trace_event_trace(struct libtrace_t *trace, struct li
 #endif
 
 	
-	if (fabs(trace->event.tdelta)>1e-9) {
-		/* Subtract the tdelta from the walltime to get a suitable
+	if (fabs(trace->event.first_ts)>1e-9) {
+		/* Subtract the tdelta from the starting times to get a suitable
 		 * "relative" time */
-		now -= trace->event.tdelta; 
+                sincebeginnow = (now - trace->event.first_now);
+                sincebegintrace = (ts - trace->event.first_ts);
 
 		/* If the trace timestamp is still in the future, return a 
 		 * SLEEP event, otherwise return the packet */
-		if (ts > now) {
-			event.seconds = ts - 
-				trace->event.trace_last_ts;
+                if (sincebeginnow <= sincebegintrace / (double)trace->replayspeedup) {
+			event.seconds = ((sincebegintrace / (double)trace->replayspeedup) - sincebeginnow);
 			event.type = TRACE_EVENT_SLEEP;
 			trace->event.waiting = true;
 			return event;
@@ -211,20 +217,10 @@ struct libtrace_eventobj_t trace_event_trace(struct libtrace_t *trace, struct li
 		 * into a timeline that is relative to the timestamps in the
 		 * trace file.
 		 */
-		trace->event.tdelta = now - ts;
+                trace->event.first_now = (double)now;
+                trace->event.first_ts = (double)ts;
 	}
 
-	/* This is the first packet, so just fire away. */
-	/* TODO: finalise packet */
-	
-	/* XXX: Could we do this more efficiently? */
-
-	/* We do a lot of freeing and creating of packet buffers with this
-	 * method, but at least it works unlike what was here previously */
-	//if (packet->buf_control == TRACE_CTRL_PACKET) {
-	//	free(packet->buffer);
-	//}
-	
 	/* The packet that we had read earlier is now ready to be returned
 	 * to the user - switch all the pointers etc. over */	
 	packet->type = trace->event.packet->type;
@@ -235,15 +231,10 @@ struct libtrace_eventobj_t trace_event_trace(struct libtrace_t *trace, struct li
 	packet->buffer = trace->event.packet->buffer;
 	packet->buf_control = trace->event.packet->buf_control;
 
-	//trace->event.packet->buffer = NULL;
-	//trace->event.packet->buf_control = TRACE_CTRL_EXTERNAL;
-	
-	//trace_destroy_packet(trace->event.packet);
-	//trace->event.packet = NULL;
+        packet->which_trace_start = trace->event.packet->which_trace_start;
 
 	event.type = TRACE_EVENT_PACKET;
 
-	trace->event.trace_last_ts = ts;
 	trace->event.waiting = false;
 
 	return event;
@@ -263,8 +254,13 @@ struct libtrace_eventobj_t trace_event_trace(struct libtrace_t *trace, struct li
 io_t *trace_open_file(libtrace_t *trace)
 {
 	io_t *io=wandio_create(trace->uridata);
+
 	if (!io) {
-		trace_set_err(trace,errno,"Unable to open %s",trace->uridata);
+		if (errno != 0) {
+			trace_set_err(trace,errno,"Unable to open %s",trace->uridata);
+		} else {
+			trace_set_err(trace,TRACE_ERR_UNSUPPORTED_COMPRESS,"Unsupported compression error: %s", trace->uridata);
+		}
 	}
 	return io;
 }
@@ -272,10 +268,28 @@ io_t *trace_open_file(libtrace_t *trace)
 /* Open a file for writing using the new Libtrace IO system */ 
 iow_t *trace_open_file_out(libtrace_out_t *trace, int compress_type, int level, int fileflag)
 {
-	assert(level<10);
-	assert(level>=0);
+	iow_t *io = NULL;
 
-	return wandio_wcreate(trace->uridata, compress_type, level, fileflag);
+        if (level < 0 || level > 9) {
+                trace_set_err_out(trace, TRACE_ERR_UNSUPPORTED_COMPRESS, 
+                                "Compression level %d is invalid, must be between 0 and 9 inclusive", 
+                                level);
+                return NULL;
+        }
+
+        if (compress_type < 0 || 
+                        compress_type >= TRACE_OPTION_COMPRESSTYPE_LAST) {
+                trace_set_err_out(trace, TRACE_ERR_UNSUPPORTED_COMPRESS,
+                                "Invalid compression type %d", compress_type);
+                return NULL;
+        }
+
+	io = wandio_wcreate(trace->uridata, compress_type, level, fileflag);
+
+	if (!io) {
+		trace_set_err_out(trace, errno, "Unable to create output file %s", trace->uridata);
+	}
+	return io;
 }
 
 
@@ -289,7 +303,12 @@ void trace_set_err(libtrace_t *trace,int errcode,const char *msg,...)
 	char buf[256];
 	va_list va;
 	va_start(va,msg);
-	assert(errcode != 0 && "An error occurred, but it is unknown what it is");
+
+	if (errcode == 0) {
+		fprintf(stderr, "An error occurred, but it is unknown what it is");
+		return;
+	}
+
 	trace->err.err_num=errcode;
 	if (errcode>0) {
 		vsnprintf(buf,sizeof(buf),msg,va);
@@ -312,7 +331,10 @@ void trace_set_err_out(libtrace_out_t *trace,int errcode,const char *msg,...)
 	char buf[256];
 	va_list va;
 	va_start(va,msg);
-	assert(errcode != 0 && "An error occurred, but it is unknown what it is");
+	if (errcode == 0) {
+		fprintf(stderr, "An error occurred, but is is unknown what is is");
+		return;
+	}
 	trace->err.err_num=errcode;
 	if (errcode>0) {
 		vsnprintf(buf,sizeof(buf),msg,va);
@@ -325,23 +347,106 @@ void trace_set_err_out(libtrace_out_t *trace,int errcode,const char *msg,...)
 	va_end(va);
 }
 
-/* Byte swapping functions for various inttypes */
-uint64_t byteswap64(uint64_t num)
-{
-	return (byteswap32((num&0xFFFFFFFF00000000ULL)>>32))
-	      |((uint64_t)byteswap32(num&0x00000000FFFFFFFFULL)<<32);
+uint32_t trace_get_number_of_cores(void) {
+
+	uint32_t t = 0;
+#ifdef _SC_NPROCESSORS_ONLN
+	t = sysconf(_SC_NPROCESSORS_ONLN);
+	if (t < 1 || t > ((uint32_t)1 << 31))
+		t = sysconf(_SC_NPROCESSORS_CONF);
+#else
+	int nm[2];
+	size_t len = 4;
+	
+	nm[0] = CTL_HW; nm[1] = HW_AVAILCPU;
+	sysctl(nm, 2, &t, &len, NULL, 0);
+	
+	if (t < 1) {
+		nm[1] = HW_NCPU;
+		sysctl(nm, 2, &t, &len, NULL, 0);
+	}
+#endif
+	if (t < 1 || t > ((uint32_t)1 << 31))
+		t = 4;
+	return t;
 }
 
-uint32_t byteswap32(uint32_t num)
-{
-	return ((num&0x000000FFU)<<24)
-		| ((num&0x0000FF00U)<<8)
-		| ((num&0x00FF0000U)>>8)
-		| ((num&0xFF000000U)>>24);
-}
-
-uint16_t byteswap16(uint16_t num)
-{
-	return ((num<<8)&0xFF00)|((num>>8)&0x00FF);
+/** Attempts to determine the direction for a pcap (or pcapng) packet.
+ *
+ * @param packet        The packet in question.
+ * @return A valid libtrace_direction_t describing the direction that the
+ *         packet was travelling, if direction can be determined. Otherwise
+ *         returns TRACE_DIR_UNKNOWN.
+ * @internal
+ *
+ * Note that we can determine the direction for only certain types of packets
+ * if they are captured using pcap/pcapng, specifically SLL and PFLOG captures.
+ */
+libtrace_direction_t pcap_get_direction(const libtrace_packet_t *packet) {
+        libtrace_direction_t direction  = -1;
+        switch(pcap_linktype_to_libtrace(rt_to_pcap_linktype(packet->type))) {
+                /* We can only get the direction for PCAP packets that have
+                 * been encapsulated in Linux SLL or PFLOG */
+                case TRACE_TYPE_LINUX_SLL:
+                {
+                        libtrace_sll_header_t *sll;
+                        libtrace_linktype_t linktype;
+
+                        sll = (libtrace_sll_header_t*)trace_get_packet_buffer(
+                                        packet,
+                                        &linktype,
+                                        NULL);
+                        if (!sll) {
+                                trace_set_err(packet->trace,
+                                        TRACE_ERR_BAD_PACKET,
+                                                "Bad or missing packet");
+                                return -1;
+                        }
+                        /* 0 == LINUX_SLL_HOST */
+                        /* the Waikato Capture point defines "packets
+                         * originating locally" (ie, outbound), with a
+                         * direction of 0, and "packets destined locally"
+                         * (ie, inbound), with a direction of 1.
+                         * This is kind-of-opposite to LINUX_SLL.
+                         * We return consistent values here, however
+                         *
+                         * Note that in recent versions of pcap, you can
+                         * use "inbound" and "outbound" on ppp in linux
+                         */
+                        if (ntohs(sll->pkttype == 0)) {
+                                direction = TRACE_DIR_INCOMING;
+                        } else {
+                                direction = TRACE_DIR_OUTGOING;
+                        }
+                        break;
+
+                }
+               case TRACE_TYPE_PFLOG:
+                {
+                        libtrace_pflog_header_t *pflog;
+                        libtrace_linktype_t linktype;
+
+                        pflog=(libtrace_pflog_header_t*)trace_get_packet_buffer(
+                                        packet,&linktype,NULL);
+                        if (!pflog) {
+                                trace_set_err(packet->trace,
+                                                TRACE_ERR_BAD_PACKET,
+                                                "Bad or missing packet");
+                                return -1;
+                        }
+                        /* enum    { PF_IN=0, PF_OUT=1 }; */
+                        if (ntohs(pflog->dir==0)) {
+
+                                direction = TRACE_DIR_INCOMING;
+                        }
+                        else {
+                                direction = TRACE_DIR_OUTGOING;
+                        }
+                        break;
+                }
+                default:
+                        break;
+        }       
+        return direction;
 }
 
