@@ -1,44 +1,35 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007,2008,2009,2010 The University of Waikato, Hamilton, 
- * New Zealand.
- *
- * Authors: Daniel Lawson 
- *          Perry Lorier
- *          Shane Alcock 
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: format_tsh.c 1611 2010-07-19 22:51:53Z salcock $
  *
  */
 
-
 #include "config.h"
 #include "common.h"
 #include "libtrace.h"
 #include "libtrace_int.h"
 #include "format_helper.h"
 
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -138,7 +129,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	if ((numbytes=wandio_read(libtrace->io,
 					buffer2,
 					(size_t)sizeof(tsh_pkt_header_t))) == -1) {
-		trace_set_err(libtrace,errno,"read(%s)",
+		trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",
 				libtrace->uridata);
 		return -1;
 	}
@@ -147,6 +138,11 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		return 0;
 	}
 
+        if (numbytes < (int)sizeof(tsh_pkt_header_t)) {
+                trace_set_err(libtrace, TRACE_ERR_WANDIO_FAILED, "Incomplete TSH header");
+                return -1;
+        }
+
 	buffer2 = (char*)buffer2 + numbytes;
 
 	/* Read the IP header */
@@ -154,7 +150,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 				buffer2,
 				(size_t)sizeof(libtrace_ip_t)+16))  /* 16 bytes of transport header */
 			!= sizeof(libtrace_ip_t)+16) {
-		trace_set_err(libtrace,errno,"read(%s)",
+		trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",
 				libtrace->uridata);
 		return -1;
 	}
@@ -169,7 +165,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	if ((numbytes=wandio_read(libtrace->io,
 				buffer2,
 				16)) != 16) {
-		trace_set_err(libtrace,errno,"read(%s)",
+		trace_set_err(libtrace,TRACE_ERR_WANDIO_FAILED,"read(%s)",
 				libtrace->uridata);
 		return -1;
 	}
@@ -230,7 +226,7 @@ static void tsh_help(void) {
 
 static struct libtrace_format_t tshformat = {
 	"tsh",
-	"$Id: format_tsh.c 1611 2010-07-19 22:51:53Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_TSH,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -246,7 +242,9 @@ static struct libtrace_format_t tshformat = {
 	tsh_read_packet,		/* read_packet */
 	tsh_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
+        NULL,                           /* can_hold_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	tsh_get_link_type,		/* get_link_type */
 	tsh_get_direction,		/* get_direction */
 	NULL,				/* set_direction */
@@ -254,6 +252,7 @@ static struct libtrace_format_t tshformat = {
 	tsh_get_timeval,		/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -264,11 +263,12 @@ static struct libtrace_format_t tshformat = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	tsh_help,			/* help */
-	NULL				/* next pointer */
+	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 /* the tsh header format is the same as tsh, except that the bits that will
@@ -278,7 +278,7 @@ static struct libtrace_format_t tshformat = {
  */
 static struct libtrace_format_t frplusformat = {
 	"fr+",
-	"$Id: format_tsh.c 1611 2010-07-19 22:51:53Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_TSH,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -294,7 +294,9 @@ static struct libtrace_format_t frplusformat = {
 	tsh_read_packet,		/* read_packet */
 	tsh_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
+        NULL,                           /* can_hold_packet */
 	NULL,				/* write_packet */
+	NULL,				/* flush_output */
 	tsh_get_link_type,		/* get_link_type */
 	tsh_get_direction,		/* get_direction */
 	NULL,				/* set_direction */
@@ -302,6 +304,7 @@ static struct libtrace_format_t frplusformat = {
 	tsh_get_timeval,		/* get_timeval */
 	NULL,				/* get_timespec */
 	NULL,				/* get_seconds */
+	NULL,                           /* get_meta_section */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
 	NULL,				/* seek_seconds */
@@ -312,11 +315,12 @@ static struct libtrace_format_t frplusformat = {
 	NULL,				/* get_received_packets */
 	NULL,				/* get_filtered_packets */
 	NULL,				/* get_dropped_packets */
-	NULL,				/* get_captured_packets */
+	NULL,				/* get_statistics */
 	NULL,				/* get_fd */
 	trace_event_trace,		/* trace_event */
 	tsh_help,			/* help */
-	NULL				/* next pointer */
+	NULL,				/* next pointer */
+	NON_PARALLEL(false)
 };
 
 void tsh_constructor(void) {
