@@ -1,3 +1,28 @@
+/*
+ *
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
+ * All rights reserved.
+ *
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
+ * research group. For further information please see http://www.wand.net.nz/
+ *
+ * libtrace is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * libtrace is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
+ */
 /* 
  * 802.11 libpacketdump decoder
  * 
@@ -24,10 +49,12 @@
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <stdio.h>
+#include <inttypes.h>
 #include "libpacketdump.h"
 #include "libtrace.h"
 
 typedef struct ieee80211_frame_control {
+# if __BYTE_ORDER == __LITTLE_ENDIAN	
 	uint8_t		version:2;
 	uint8_t		type:2;
 	uint8_t		subtype:4;
@@ -39,6 +66,21 @@ typedef struct ieee80211_frame_control {
         uint8_t		more_data:1;
         uint8_t		wep:1;
         uint8_t		order:1;
+# elif __BYTE_ORDER == __BIG_ENDIAN
+	uint8_t		subtype:4;
+	uint8_t		type:2;
+	uint8_t		version:2;
+        uint8_t		order:1;
+        uint8_t		wep:1;
+        uint8_t		more_data:1;
+        uint8_t		power:1;
+        uint8_t		retry:1;
+	uint8_t		more_frag:1;
+	uint8_t		from_ds:1;
+	uint8_t		to_ds:1;
+#else
+#	error "Adjust your <bits/endian.h> defines"
+# endif	
 } __attribute__ ((__packed__)) ieee80211_frame_control;
 
 typedef struct ieee80211_ctrl_frame_1addr {
@@ -105,22 +147,43 @@ static char *macaddr(uint8_t mac[]) {
 }
 
 typedef struct ieee80211_capinfo {
-	uint16_t	ess:1;
-	uint16_t	ibss:1;
-	uint16_t	cf_pollable:1;
-	uint16_t	cf_poll_req:1;
-	uint16_t	privacy:1;
-	uint16_t	short_preamble:1;
-	uint16_t	pbcc:1;
-	uint16_t	channel_agility:1;
-	uint16_t	spectrum_mgmt:1;
-	uint16_t	qos:1;
-	uint16_t	short_slot_time:1;
-	uint16_t	apsd:1;
-	uint16_t	res1:1;
-	uint16_t	dsss_ofdm:1;
-	uint16_t	delayed_block_ack:1;
-	uint16_t	immediate_block_ack:1;
+#if __BYTE_ORDER == __LITTLE_ENDIAN 
+	uint8_t	ess:1;
+	uint8_t	ibss:1;
+	uint8_t	cf_pollable:1;
+	uint8_t	cf_poll_req:1;
+	uint8_t	privacy:1;
+	uint8_t	short_preamble:1;
+	uint8_t	pbcc:1;
+	uint8_t	channel_agility:1;
+	uint8_t	spectrum_mgmt:1;
+	uint8_t	qos:1;
+	uint8_t	short_slot_time:1;
+	uint8_t	apsd:1;
+	uint8_t	res1:1;
+	uint8_t	dsss_ofdm:1;
+	uint8_t	delayed_block_ack:1;
+	uint8_t	immediate_block_ack:1;
+#elif __BYTE_ORDER == __BIG_ENDIAN
+	uint8_t	channel_agility:1;
+	uint8_t	pbcc:1;
+	uint8_t	short_preamble:1;
+	uint8_t	privacy:1;
+	uint8_t	cf_poll_req:1;
+	uint8_t	cf_pollable:1;
+	uint8_t	ibss:1;
+	uint8_t	ess:1;
+	uint8_t	immediate_block_ack:1;
+	uint8_t	delayed_block_ack:1;
+	uint8_t	dsss_ofdm:1;
+	uint8_t	res1:1;
+	uint8_t	apsd:1;
+	uint8_t	short_slot_time:1;
+	uint8_t	qos:1;
+	uint8_t	spectrum_mgmt:1;
+#else
+# error "Unknown byte order -- please check <bits/endian.h>"
+#endif
 } __attribute__ ((__packed__)) ieee80211_capinfo;
 
 typedef struct ieee80211_beacon {
@@ -193,7 +256,7 @@ static void decode_80211_vendor_ie(ieee80211_ie *ie) {
 static void decode_80211_information_elements(const char *pkt, unsigned len) {
 	ieee80211_ie *ie;
 	int i = 0;
-	uint8_t * data;
+	const uint8_t * data;
 	uint8_t bmap_offset;
 	while (len >= sizeof(ieee80211_ie)) {
 		ie = (ieee80211_ie *) pkt;
@@ -203,7 +266,7 @@ static void decode_80211_information_elements(const char *pkt, unsigned len) {
 			return;
 		}
 		
-		data = (( char *)pkt + sizeof (ieee80211_ie));
+		data = (( const unsigned char *)pkt + sizeof (ieee80211_ie));
 		
 		switch (ie->id) {
 			case 0:
@@ -431,7 +494,7 @@ static void decode_80211_beacon(const char *pkt, unsigned len) {
 		return;
 	}
 	
-	printf(" 802.11MAC: Timestamp = %llu\n", b->ts);
+	printf(" 802.11MAC: Timestamp = %" PRIu64 "\n", b->ts);
 	printf(" 802.11MAC: Beacon Interval = %u\n", b->interval);
 	decode_80211_capinfo(&b->capinfo);
 	printf(" 802.11MAC: Information Elements:\n");
@@ -498,7 +561,7 @@ static void decode_80211_authentication_frame(const char *pkt, unsigned len) {
 
 static void decode_80211_mgmt(const char *pkt, unsigned len) {
 	ieee80211_mgmt_frame *mgmt = (ieee80211_mgmt_frame *)pkt;
-	uint8_t *data;
+	const char *data;
 	
 	printf(" 802.11MAC: Management frame: ");
 	
@@ -556,7 +619,7 @@ static void decode_80211_mgmt(const char *pkt, unsigned len) {
 			decode_80211_beacon(pkt, len);
 			break;
 		case 10:
-			data = ((char *)pkt + sizeof(ieee80211_mgmt_frame));
+			data = (pkt + sizeof(ieee80211_mgmt_frame));
 			printf(" 802.11MAC: Reason Code = ");
 			ieee80211_print_reason_code((uint16_t) ((data[0] << 8) | (data[1])));
 			printf("\n");
@@ -566,7 +629,7 @@ static void decode_80211_mgmt(const char *pkt, unsigned len) {
 			decode_80211_authentication_frame(pkt, len);
 			break;
 		case 12:
-			data = ((char *)pkt + sizeof(ieee80211_mgmt_frame));
+			data = (pkt + sizeof(ieee80211_mgmt_frame));
 			printf(" 802.11MAC: Reason Code = ");
 			ieee80211_print_reason_code((uint16_t) ((data[0] << 8) | (data[1])));
 			printf("\n");
@@ -746,7 +809,7 @@ static void decode_80211_data(const char *pkt, unsigned len) {
 	
 }
 
-void decode(int link_type, const char *pkt, unsigned len) 
+DLLEXPORT void decode(int link_type UNUSED, const char *pkt, unsigned len) 
 {
 	ieee80211_frame_control *fc;
 	
@@ -784,7 +847,7 @@ void decode(int link_type, const char *pkt, unsigned len)
 			break;
 		case 3:
 			printf(" Unable to decode frame type %u, dumping rest of packet\n", fc->type);
-			decode_next(pkt + sizeof(ieee80211_frame_control), len - sizeof(ieee80211_frame_control), "eth", 0);
+			decode_next(pkt + sizeof(ieee80211_frame_control), len - sizeof(ieee80211_frame_control), "unknown", 0);
 			
 			break;
 	}
