@@ -1,34 +1,30 @@
 /*
- * This file is part of libtrace
  *
- * Copyright (c) 2007 The University of Waikato, Hamilton, New Zealand.
- * Authors: Daniel Lawson 
- *          Perry Lorier 
- *          Josef Vodanovich
- *          
+ * Copyright (c) 2007-2016 The University of Waikato, Hamilton, New Zealand.
  * All rights reserved.
  *
- * This code has been developed by the University of Waikato WAND 
+ * This file is part of libtrace.
+ *
+ * This code has been developed by the University of Waikato WAND
  * research group. For further information please see http://www.wand.net.nz/
  *
  * libtrace is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * libtrace is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with libtrace; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: tracereport.c 1551 2010-03-29 02:49:08Z salcock $
  *
  */
 
+
 /* This program takes a series of traces and bpf filters and outputs how many
  * bytes/packets
  */
@@ -57,6 +53,7 @@ static volatile int done=0;
 static void cleanup_signal(int sig UNUSED)
 {
 	done=1;
+	trace_interrupt();
 }
 
 /* Process a trace, counting packets that match filter(s) */
@@ -95,6 +92,9 @@ static void run_trace(char *uri, libtrace_filter_t *filter, int count)
 		if ((psize = trace_read_packet(trace, packet)) <1) {
 			break;
 		}
+                if (IS_LIBTRACE_META_PACKET(packet))
+                        continue;
+
 		if (reports_required & REPORT_TYPE_MISC)
 			misc_per_packet(packet);
 		if (reports_required & REPORT_TYPE_ERROR)
@@ -126,6 +126,7 @@ static void run_trace(char *uri, libtrace_filter_t *filter, int count)
 	}
 	if (reports_required & REPORT_TYPE_DROPS)
 		drops_per_trace(trace);
+        trace_destroy_packet(packet);
 	trace_destroy(trace);
 }
 
